"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const open_1 = __importDefault(require("open"));
const _1 = require(".");
const config_1 = require("../config");
const server_1 = __importDefault(require("../server"));
let processExistCode = 0;
class StartCommandCommand extends command_1.Command {
    static description = 'StartCommands uniflow client.';
    static examples = [`$ uniflow-client start`, `$ uniflow-client start -o`];
    static flags = {
        help: command_1.flags.help({ char: 'h' }),
        open: command_1.flags.boolean({
            char: 'o',
            description: 'opens the UI automatically in browser',
        }),
    };
    /**
     * Opens the UI in browser
     */
    static openBrowser() {
        const url = `http://localhost:${config_1.appConfig.get('port')}`;
        (0, open_1.default)(url, { wait: true }).catch((error) => {
            console.log(`\nWas not able to open URL in browser. Please open manually by visiting:\n${url}\n`);
        });
    }
    /**
     * Stoppes uniflow in a graceful way.
     */
    static async stopProcess() {
        console.log(`\nStopping uniflow...`);
        setTimeout(() => {
            // In case that something goes wrong with shutdown we
            // kill after max. 30 seconds no matter what
            process.exit(processExistCode);
        }, 30000);
        process.exit(processExistCode);
    }
    async run() {
        // Make sure that uniflow shuts down gracefully if possible
        process.on('SIGTERM', _1.StartCommand.stopProcess);
        process.on('SIGINT', _1.StartCommand.stopProcess);
        const { flags } = this.parse(_1.StartCommand);
        // Wrap that the process does not close but we can still use async
        (async () => {
            try {
                await (0, server_1.default)();
                const url = `http://localhost:${config_1.appConfig.get('port')}`;
                this.log(`\nUniflow client v${require('../../../../package.json').version} is ready on:\n${url}`);
                // Allow to open uniflow editor by pressing "o"
                if (Boolean(process.stdout.isTTY) && process.stdin.setRawMode) {
                    process.stdin.setRawMode(true);
                    process.stdin.resume();
                    process.stdin.setEncoding('utf8');
                    let inputText = '';
                    if (flags.open) {
                        _1.StartCommand.openBrowser();
                    }
                    this.log(`\nPress "o" to open in Browser.`);
                    process.stdin.on('data', (key) => {
                        if (key === 'o') {
                            _1.StartCommand.openBrowser();
                            inputText = '';
                        }
                        else if (key.charCodeAt(0) === 3) {
                            // Ctrl + c got pressed
                            _1.StartCommand.stopProcess();
                        }
                        else {
                            // When anything else got pressed, record it and send it on enter into the child process
                            if (key.charCodeAt(0) === 13) {
                                // send to child process and print in terminal
                                process.stdout.write('\n');
                                inputText = '';
                            }
                            else {
                                // record it and write into terminal
                                inputText += key;
                                process.stdout.write(key);
                            }
                        }
                    });
                }
            }
            catch (error) {
                if (error instanceof Error) {
                    this.error(`There was an error: ${error.message}`);
                }
                processExistCode = 1;
                // @ts-ignore
                process.emit('SIGINT');
            }
        })();
    }
}
exports.default = StartCommandCommand;
