"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormInputType = void 0;
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const react_1 = __importDefault(require("react"));
const checkbox_1 = __importDefault(require("./checkbox"));
const editor_1 = __importDefault(require("./editor"));
const select_1 = __importDefault(require("./select"));
var FormInputType;
(function (FormInputType) {
    FormInputType["TEXT"] = "TEXT";
    FormInputType["TEXTAREA"] = "TEXTAREA";
    FormInputType["EDITOR"] = "EDITOR";
    FormInputType["PASSWORD"] = "PASSWORD";
    FormInputType["CHECKBOX"] = "CHECKBOX";
    FormInputType["SELECT"] = "SELECT";
})(FormInputType = exports.FormInputType || (exports.FormInputType = {}));
const InputGroups = ({ children, icon, groups, errors }) => {
    const errorMessages = errors && errors.map((message, i) => (react_1.default.createElement("div", { key: `error-${i}`, className: "invalid-feedback" }, message)));
    return icon || groups.length > 0 ? (react_1.default.createElement("div", { className: "input-group" },
        icon && (react_1.default.createElement("div", { className: "input-group-text" },
            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: icon }))),
        groups,
        children,
        errorMessages)) : (react_1.default.createElement(react_1.default.Fragment, null,
        children,
        errorMessages));
};
const FormInput = (props) => {
    const { type, id, label, value, errors, icon, onChange, autoComplete, rows, language, multiple, options } = props;
    const placeholder = props.placeholder || label;
    const groups = props.groups ? (Array.isArray(props.groups) ? props.groups : [react_1.default.createElement(react_1.default.Fragment, { key: "item" }, props.groups)]) : [];
    const onChangeInput = (event) => {
        if (onChange) {
            onChange(event.target.value);
        }
    };
    return (react_1.default.createElement("div", { className: "row mb-3" },
        label && (react_1.default.createElement("label", { htmlFor: "settings-lastname", className: "col-sm-2 col-form-label" }, label)),
        react_1.default.createElement("div", { className: label ? 'col-sm-10' : 'col-sm-12' },
            react_1.default.createElement(InputGroups, { icon: icon, groups: groups, errors: errors },
                (type === FormInputType.TEXT || type === FormInputType.PASSWORD) && (react_1.default.createElement("input", { className: `form-control${errors ? ' is-invalid' : ''}`, id: id, type: type === FormInputType.PASSWORD ? 'password' : 'text', value: value || '', onChange: onChangeInput, placeholder: placeholder, autoComplete: autoComplete ? id : undefined })),
                type === FormInputType.TEXTAREA && (react_1.default.createElement("textarea", { className: `form-control${errors ? ' is-invalid' : ''}`, id: id, value: value || '', onChange: onChangeInput, placeholder: placeholder, rows: rows })),
                type === FormInputType.EDITOR && (react_1.default.createElement("div", { className: `form-control-plaintext${errors ? ' is-invalid' : ''}` },
                    react_1.default.createElement(editor_1.default, { id: id, value: value || '', onChange: onChange, language: language, readonly: false }))),
                type === FormInputType.CHECKBOX && (react_1.default.createElement(checkbox_1.default, { id: id, className: `form-control-plaintext${errors ? ' is-invalid' : ''}`, value: value, onChange: onChange })),
                type === FormInputType.SELECT && (react_1.default.createElement(select_1.default, { id: id, value: value, onChange: onChange, className: `form-control${errors ? ' is-invalid' : ''}`, multiple: multiple, options: options }))))));
};
exports.default = FormInput;
