"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_select_1 = __importDefault(require("react-select"));
const creatable_1 = __importDefault(require("react-select/creatable"));
const app_1 = require("../contexts/app");
const Select = (props) => {
    const { value, options, edit, multiple } = props;
    const app = (0, app_1.useApp)();
    const customStyles = {
        light: {
            menu: (provided) => ({
                ...provided,
                zIndex: 10,
            }),
            multiValue: (styles) => ({
                ...styles,
                backgroundColor: '#6c757d',
            }),
            multiValueLabel: (styles) => ({
                ...styles,
                color: '#FFFFFF',
            }),
        },
        dark: {
            menu: (provided) => ({
                ...provided,
                zIndex: 10,
            }),
            multiValue: (styles) => ({
                ...styles,
                backgroundColor: '#576068',
            }),
            multiValueLabel: (styles) => ({
                ...styles,
                color: '#ebf4f1',
            }),
        },
        sepia: {
            menu: (provided) => ({
                ...provided,
                zIndex: 10,
            }),
            multiValue: (styles) => ({
                ...styles,
                backgroundColor: '#876944',
            }),
            multiValueLabel: (styles) => ({
                ...styles,
                color: '#eadec2',
            }),
        },
    };
    const themes = {
        light: (theme) => theme,
        dark: (theme) => ({
            ...theme,
            colors: {
                ...theme.colors,
                primary25: '#0056b3',
                neutral0: '#0e2233',
                neutral5: '#0f2b3d',
                neutral10: '#576068',
                neutral20: '#002651',
                neutral70: '#002651',
                neutral80: '#495057',
                dangerLight: '#d59b8b',
            },
        }),
        sepia: (theme) => ({
            ...theme,
            colors: {
                ...theme.colors,
                primary25: '#0056b3',
                neutral0: '#eadec2',
                neutral5: '#0f2b3d',
                neutral10: '#876944',
                neutral20: '#ded0bf',
                neutral70: '#002651',
                neutral80: '#495057',
                dangerLight: '#d59b8b',
            },
        }),
    };
    const selectOptions = options || [];
    let selectValue = undefined;
    if (multiple === true) {
        selectValue = [];
        if (value && Array.isArray(value)) {
            selectValue = value.map((data) => {
                return { value: data, label: data };
            });
        }
    }
    else {
        selectValue = selectOptions.filter((option) => {
            return option.value === value;
        });
    }
    const DisplaySelect = edit === true ? creatable_1.default : react_select_1.default;
    return (react_1.default.createElement(DisplaySelect, { isMulti: multiple === true, value: selectValue, onChange: (data) => {
            const { onChange, multiple } = props;
            if (onChange) {
                if (multiple === true && Array.isArray(data)) {
                    onChange((data || []).map((option) => {
                        return option.value;
                    }));
                }
                else if (!Array.isArray(data)) {
                    onChange(data.value);
                }
            }
        }, options: selectOptions, styles: customStyles[app.theme], theme: themes[app.theme] }));
};
exports.default = Select;
