"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAuth = exports.AuthConsumer = exports.AuthProvider = exports.AuthContext = exports.register = exports.githubLogin = exports.githubLoginUrl = exports.facebookLogin = exports.facebookLoginUrl = exports.login = exports.commitLogout = exports.commitLoginFailure = exports.commitLoginSuccess = exports.commitLoginRequest = exports.AuthActionTypes = void 0;
const react_1 = __importStar(require("react"));
const container_1 = __importDefault(require("../container"));
const services_1 = require("../services");
const jwt_decode_1 = __importStar(require("jwt-decode"));
const use_reducer_ref_1 = require("../hooks/use-reducer-ref");
const container = new container_1.default();
const api = container.get(services_1.Api);
var AuthActionTypes;
(function (AuthActionTypes) {
    AuthActionTypes["COMMIT_LOGIN_REQUEST"] = "COMMIT_LOGIN_REQUEST";
    AuthActionTypes["COMMIT_LOGIN_SUCCESS"] = "COMMIT_LOGIN_SUCCESS";
    AuthActionTypes["COMMIT_LOGIN_FAILURE"] = "COMMIT_LOGIN_FAILURE";
    AuthActionTypes["COMMIT_LOGOUT"] = "COMMIT_LOGOUT";
})(AuthActionTypes = exports.AuthActionTypes || (exports.AuthActionTypes = {}));
const defaultState = (() => {
    if (typeof window !== 'undefined') {
        const token = window.localStorage.getItem('token');
        const uid = window.localStorage.getItem('uid');
        if (token && uid) {
            return {
                token: token,
                uid: uid,
                isAuthenticated: true,
                isAuthenticating: false,
                message: undefined,
            };
        }
    }
    return {
        token: undefined,
        uid: undefined,
        isAuthenticated: false,
        isAuthenticating: false,
        message: undefined,
    };
})();
const commitLoginRequest = () => {
    return (dispatch) => {
        dispatch({
            type: AuthActionTypes.COMMIT_LOGIN_REQUEST,
        });
    };
};
exports.commitLoginRequest = commitLoginRequest;
const commitLoginSuccess = (token, uid) => {
    return (dispatch) => {
        if (typeof window !== `undefined`) {
            window.localStorage.setItem('token', token);
            window.localStorage.setItem('uid', uid);
        }
        dispatch({
            type: AuthActionTypes.COMMIT_LOGIN_SUCCESS,
            token,
            uid,
        });
    };
};
exports.commitLoginSuccess = commitLoginSuccess;
const commitLoginFailure = (message) => {
    return (dispatch) => {
        if (typeof window !== `undefined`) {
            window.localStorage.removeItem('token');
            window.localStorage.removeItem('uid');
        }
        dispatch({
            type: AuthActionTypes.COMMIT_LOGIN_FAILURE,
            message,
        });
    };
};
exports.commitLoginFailure = commitLoginFailure;
const commitLogout = () => {
    return (dispatch) => {
        if (typeof window !== `undefined`) {
            window.localStorage.removeItem('token');
            window.localStorage.removeItem('uid');
        }
        dispatch({
            type: AuthActionTypes.COMMIT_LOGOUT,
        });
    };
};
exports.commitLogout = commitLogout;
const login = (username, password) => {
    return async (dispatch) => {
        (0, exports.commitLoginRequest)()(dispatch);
        try {
            const data = await api.login({ username, password });
            (0, jwt_decode_1.default)(data.token);
            (0, exports.commitLoginSuccess)(data.token, data.uid)(dispatch);
        }
        catch (error) {
            if (error instanceof jwt_decode_1.InvalidTokenError) {
                (0, exports.commitLoginFailure)('Invalid token')(dispatch);
            }
            else {
                (0, exports.commitLoginFailure)('Invalid credentials')(dispatch);
            }
            throw error;
        }
    };
};
exports.login = login;
const facebookLoginUrl = (facebookAppId) => {
    const httphost = typeof window !== `undefined` ? `${window.location.origin}` : '';
    return `https://www.facebook.com/v3.2/dialog/oauth?client_id=${facebookAppId}&response_type=token&redirect_uri=${httphost}/login/facebook`;
};
exports.facebookLoginUrl = facebookLoginUrl;
const facebookLogin = (accessToken, token) => {
    return async (dispatch) => {
        (0, exports.commitLoginRequest)()(dispatch);
        try {
            const data = await api.loginFacebook({ access_token: accessToken }, { token });
            (0, jwt_decode_1.default)(data.token);
            (0, exports.commitLoginSuccess)(data.token, data.uid)(dispatch);
        }
        catch (error) {
            if (error instanceof jwt_decode_1.InvalidTokenError) {
                (0, exports.commitLoginFailure)('Invalid token')(dispatch);
            }
            else {
                (0, exports.commitLoginFailure)('Invalid credentials')(dispatch);
            }
            throw error;
        }
    };
};
exports.facebookLogin = facebookLogin;
const githubLoginUrl = (githubAppId) => {
    const httphost = typeof window !== `undefined` ? `${window.location.origin}` : '';
    return `https://github.com/login/oauth/authorize?client_id=${githubAppId}&redirect_uri=${httphost}/login/github`;
};
exports.githubLoginUrl = githubLoginUrl;
const githubLogin = (code, token) => {
    return async (dispatch) => {
        (0, exports.commitLoginRequest)()(dispatch);
        try {
            const data = await api.loginGithub({ code }, { token });
            (0, jwt_decode_1.default)(data.token);
            (0, exports.commitLoginSuccess)(data.token, data.uid)(dispatch);
        }
        catch (error) {
            if (error instanceof jwt_decode_1.InvalidTokenError) {
                (0, exports.commitLoginFailure)('Invalid token')(dispatch);
            }
            else {
                (0, exports.commitLoginFailure)('Invalid credentials')(dispatch);
            }
            throw error;
        }
    };
};
exports.githubLogin = githubLogin;
const register = (email, password) => {
    return async (dispatch) => {
        (0, exports.commitLoginRequest)()(dispatch);
        try {
            await api.createUser({ email, password });
            await (0, exports.login)(email, password)(dispatch);
        }
        catch (error) {
            if (error instanceof jwt_decode_1.InvalidTokenError) {
                (0, exports.commitLoginFailure)('Invalid token')(dispatch);
            }
            else {
                (0, exports.commitLoginFailure)('Invalid credentials')(dispatch);
            }
            throw error;
        }
    };
};
exports.register = register;
exports.AuthContext = react_1.default.createContext({
    auth: defaultState,
    authDispatch: () => {
        throw new Error('AuthContext not yet initialized.');
    },
    authRef: {
        current: defaultState,
    },
});
exports.AuthContext.displayName = 'AuthContext';
const AuthProvider = (props) => {
    const [auth, authDispatch, authRef] = (0, use_reducer_ref_1.useReducerRef)((state = defaultState, action) => {
        switch (action.type) {
            case AuthActionTypes.COMMIT_LOGIN_REQUEST:
                return Object.assign({}, state, {
                    isAuthenticating: true,
                    message: undefined,
                });
            case AuthActionTypes.COMMIT_LOGIN_SUCCESS:
                return Object.assign({}, state, {
                    isAuthenticating: false,
                    isAuthenticated: true,
                    token: action.token,
                    uid: action.uid,
                    message: 'You have been successfully logged in.',
                });
            case AuthActionTypes.COMMIT_LOGIN_FAILURE:
                return Object.assign({}, state, {
                    isAuthenticating: false,
                    isAuthenticated: false,
                    token: undefined,
                    uid: undefined,
                    message: `Authentication Error: ${action.message}`,
                });
            case AuthActionTypes.COMMIT_LOGOUT:
                return Object.assign({}, state, {
                    isAuthenticated: false,
                    token: undefined,
                    uid: undefined,
                    message: 'You have been successfully logged out.',
                });
            default:
                return state;
        }
    }, defaultState);
    return (react_1.default.createElement(exports.AuthContext.Provider, { value: { auth, authDispatch, authRef } }, props.children));
};
exports.AuthProvider = AuthProvider;
exports.AuthConsumer = exports.AuthContext.Consumer;
function useAuth() {
    return (0, react_1.useContext)(exports.AuthContext);
}
exports.useAuth = useAuth;
