"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFeed = exports.FeedConsumer = exports.FeedProvider = exports.FeedContext = exports.deleteParentFolder = exports.updateParentFolder = exports.createFolder = exports.getFolderTree = exports.deleteProgram = exports.setProgramData = exports.getProgramData = exports.updateProgram = exports.createProgram = exports.fetchFeed = exports.getOrderedFeed = exports.getTags = exports.toFeedPath = exports.getFeedItem = exports.commitSetUidFeed = exports.commitSetSlugFeed = exports.commitSetParentFolderFeed = exports.commitDeleteFeed = exports.commitUpdateFeed = exports.commitClearFeed = exports.apiToPaginatedProgramFeedEntity = exports.apiToProgramFeedEntity = exports.apiToPaginatedFolderFeedEntity = exports.apiToFolderFeedEntity = exports.FeedActionTypes = void 0;
const react_1 = __importStar(require("react"));
const container_1 = __importDefault(require("../container"));
const services_1 = require("../services");
const moment_1 = __importDefault(require("moment"));
const user_1 = require("./user");
const routes_1 = require("../routes");
const use_reducer_ref_1 = require("../hooks/use-reducer-ref");
const api_exceptions_1 = require("../models/api-exceptions");
const container = new container_1.default();
const api = container.get(services_1.Api);
var FeedActionTypes;
(function (FeedActionTypes) {
    FeedActionTypes["COMMIT_CLEAR_FEED"] = "COMMIT_CLEAR_FEED";
    FeedActionTypes["COMMIT_UPDATE_FEED"] = "COMMIT_UPDATE_FEED";
    FeedActionTypes["COMMIT_DELETE_FEED"] = "COMMIT_DELETE_FEED";
    FeedActionTypes["COMMIT_SET_PARENT_FOLDER_FEED"] = "COMMIT_SET_PARENT_FOLDER_FEED";
    FeedActionTypes["COMMIT_SET_SLUG_FEED"] = "COMMIT_SET_SLUG_FEED";
    FeedActionTypes["COMMIT_SET_UID_FEED"] = "COMMIT_SET_UID_FEED";
})(FeedActionTypes = exports.FeedActionTypes || (exports.FeedActionTypes = {}));
const defaultState = {
    items: {},
    parentFolder: undefined,
    slug: undefined,
    uid: undefined,
};
const apiToFolderFeedEntity = (apiEntity) => {
    return {
        ...apiEntity,
        ...{
            created: (0, moment_1.default)(apiEntity.created),
            updated: (0, moment_1.default)(apiEntity.updated),
        }
    };
};
exports.apiToFolderFeedEntity = apiToFolderFeedEntity;
const apiToPaginatedFolderFeedEntity = (apiPaginationEntity) => {
    const { data, total } = apiPaginationEntity;
    return {
        data: data.map((apiEntity) => (0, exports.apiToFolderFeedEntity)(apiEntity)),
        total
    };
};
exports.apiToPaginatedFolderFeedEntity = apiToPaginatedFolderFeedEntity;
const apiToProgramFeedEntity = (apiEntity) => {
    return {
        ...apiEntity,
        ...{
            data: null,
            created: (0, moment_1.default)(apiEntity.created),
            updated: (0, moment_1.default)(apiEntity.updated),
        }
    };
};
exports.apiToProgramFeedEntity = apiToProgramFeedEntity;
const apiToPaginatedProgramFeedEntity = (apiPaginationEntity) => {
    const { data, total } = apiPaginationEntity;
    return {
        data: data.map((apiEntity) => (0, exports.apiToProgramFeedEntity)(apiEntity)),
        total
    };
};
exports.apiToPaginatedProgramFeedEntity = apiToPaginatedProgramFeedEntity;
const fetchCollection = async (uid, type, path, page = 1, token, items = []) => {
    let paginedEntities;
    if (type === 'programs') {
        paginedEntities = (0, exports.apiToPaginatedProgramFeedEntity)(await api.getUserPrograms({ uid }, { page, path }, { token }));
    }
    else if (type === 'folders') {
        paginedEntities = (0, exports.apiToPaginatedFolderFeedEntity)(await api.getUserFolders({ uid }, { page, path }, { token }));
    }
    if (paginedEntities) {
        const { data, total } = paginedEntities;
        items = items.concat(data);
        if (items.length < total) {
            return await fetchCollection(uid, type, path, page + 1, token, items);
        }
    }
    return items;
};
const commitClearFeed = () => {
    return (dispatch) => {
        dispatch({
            type: FeedActionTypes.COMMIT_CLEAR_FEED,
        });
    };
};
exports.commitClearFeed = commitClearFeed;
const commitUpdateFeed = (item) => {
    return (dispatch) => {
        dispatch({
            type: FeedActionTypes.COMMIT_UPDATE_FEED,
            item,
        });
    };
};
exports.commitUpdateFeed = commitUpdateFeed;
const commitDeleteFeed = (item) => {
    return (dispatch) => {
        dispatch({
            type: FeedActionTypes.COMMIT_DELETE_FEED,
            item,
        });
    };
};
exports.commitDeleteFeed = commitDeleteFeed;
const commitSetParentFolderFeed = (parentFolder) => {
    return (dispatch) => {
        dispatch({
            type: FeedActionTypes.COMMIT_SET_PARENT_FOLDER_FEED,
            parentFolder,
        });
    };
};
exports.commitSetParentFolderFeed = commitSetParentFolderFeed;
const commitSetSlugFeed = (slug) => {
    return (dispatch) => {
        dispatch({
            type: FeedActionTypes.COMMIT_SET_SLUG_FEED,
            slug,
        });
    };
};
exports.commitSetSlugFeed = commitSetSlugFeed;
const commitSetUidFeed = (uid) => {
    return (dispatch) => {
        dispatch({
            type: FeedActionTypes.COMMIT_SET_UID_FEED,
            uid,
        });
    };
};
exports.commitSetUidFeed = commitSetUidFeed;
const getFeedItem = (feed, slug = feed.slug) => {
    if (slug === undefined)
        return undefined;
    if (slug === null && feed.parentFolder) {
        return {
            type: 'folder',
            entity: feed.parentFolder,
        };
    }
    for (const item of Object.values(feed.items)) {
        if (item.entity.slug === slug) {
            return item;
        }
    }
    return undefined;
};
exports.getFeedItem = getFeedItem;
const toFeedPath = (entity, user, toParent = false) => {
    const paths = entity.path === '/' ? [] : entity.path.split('/').slice(1);
    if (!toParent && entity.slug) {
        paths.push(entity.slug);
    }
    const slugs = {};
    for (let i = 0; i < paths.length; i++) {
        slugs[`slug${i + 1}`] = paths[i];
    }
    const isCurrentUser = entity.user === user.uid || entity.user === user.username;
    if (isCurrentUser) {
        return (0, routes_1.pathTo)('feed', slugs);
    }
    return (0, routes_1.pathTo)('userFeed', Object.assign({ uid: entity.user }, slugs));
};
exports.toFeedPath = toFeedPath;
const getTags = (feed) => {
    let tags = Object.keys(feed.items).reduce(function (currentTags, key) {
        const entity = feed.items[key].entity;
        const tags = 'tags' in entity ? entity.tags : [];
        return currentTags.concat();
    }, []);
    // filter unique
    tags = tags.filter(function (value, index, self) {
        return self.indexOf(value) === index;
    });
    return tags;
};
exports.getTags = getTags;
const getOrderedFeed = (feed, filter) => {
    let keys = Object.keys(feed.items);
    if (filter !== undefined) {
        keys = keys.filter((key) => {
            const item = feed.items[key];
            const words = [];
            if (item.type === 'program') {
                words.push(item.entity.name);
                const tags = 'tags' in item.entity ? item.entity.tags : [];
                for (let i = 0; i < tags.length; i++) {
                    words.push(tags[i]);
                }
            }
            else if (item.type === 'folder') {
                words.push(item.entity.name);
            }
            const searchWords = words.join(' ').toLowerCase();
            return searchWords.indexOf(filter) !== -1;
        });
    }
    keys.sort((keyA, keyB) => {
        const itemA = feed.items[keyA];
        const itemB = feed.items[keyB];
        return (0, moment_1.default)(itemB.entity.updated).diff((0, moment_1.default)(itemA.entity.updated));
    });
    return keys.map((key) => {
        return feed.items[key];
    });
};
exports.getOrderedFeed = getOrderedFeed;
const fetchFeed = (uid, paths, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        const path = `/${paths.join('/')}`;
        const parentPath = `/${paths.slice(0, -1).join('/')}`;
        const parentParentPath = `/${paths.slice(0, -2).join('/')}`;
        const slug = paths.length > 0 ? paths[paths.length - 1] : null;
        const parentSlug = paths.length > 1 ? paths[paths.length - 2] : null;
        let folderPathFeed = parentPath;
        let feedProgramPath = parentPath;
        let parentFolderFound = undefined;
        (0, exports.commitSetUidFeed)(uid)(dispatch);
        (0, exports.commitSetSlugFeed)(slug)(dispatch);
        (0, exports.commitSetParentFolderFeed)(null)(dispatch);
        (0, exports.commitClearFeed)()(dispatch);
        if (paths.length > 0) {
            const folders = await fetchCollection(uid, 'folders', parentPath, 1, token);
            parentFolderFound = false;
            for (const folder of folders) {
                if (folder.path === parentPath && folder.slug === slug) {
                    (0, exports.commitSetSlugFeed)(null)(dispatch);
                    (0, exports.commitSetParentFolderFeed)(folder)(dispatch);
                    folderPathFeed = path;
                    feedProgramPath = path;
                    parentFolderFound = true;
                }
            }
            if (parentFolderFound === false) {
                for (const folder of folders) {
                    (0, exports.commitUpdateFeed)({
                        type: 'folder',
                        entity: folder,
                    })(dispatch);
                }
                folderPathFeed = parentParentPath;
            }
        }
        try {
            const [folders, programs] = await Promise.all([
                fetchCollection(uid, 'folders', folderPathFeed, 1, token),
                fetchCollection(uid, 'programs', feedProgramPath, 1, token),
            ]);
            if (parentFolderFound === true || parentFolderFound === undefined) {
                for (const folder of folders) {
                    (0, exports.commitUpdateFeed)({
                        type: 'folder',
                        entity: folder,
                    })(dispatch);
                }
            }
            if (parentFolderFound === false) {
                for (const folder of folders) {
                    if (folder.path === parentParentPath && folder.slug === parentSlug) {
                        (0, exports.commitSetParentFolderFeed)(folder)(dispatch);
                    }
                }
            }
            for (const program of programs) {
                (0, exports.commitUpdateFeed)({
                    type: 'program',
                    entity: program,
                })(dispatch);
            }
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.fetchFeed = fetchFeed;
const createProgram = (program, uid, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        const data = {
            name: program.name,
            ...(program.slug && { slug: program.slug }),
            path: program.path,
            clients: program.clients,
            tags: program.tags,
            description: program.description,
        };
        try {
            const programFeed = (0, exports.apiToProgramFeedEntity)(await api.createUserProgram({ uid }, data, { token }));
            (0, exports.commitUpdateFeed)({
                type: 'program',
                entity: programFeed,
            })(dispatch);
            return programFeed;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.createProgram = createProgram;
const updateProgram = (program, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        const data = {
            name: program.name,
            slug: program.slug,
            path: program.path,
            clients: program.clients,
            tags: program.tags,
            description: program.description,
            isPublic: program.isPublic,
        };
        try {
            program = (0, exports.apiToProgramFeedEntity)(await api.updateProgram({ uid: program.uid }, data, { token }));
            (0, exports.commitUpdateFeed)({
                type: 'program',
                entity: program,
            })(dispatch);
            return program;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.updateProgram = updateProgram;
const getProgramData = (program, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        try {
            const data = await api.getProgramFlows({ uid: program.uid }, { token });
            return data.data;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.getProgramData = getProgramData;
const setProgramData = (program, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        const body = {
            data: program.data,
        };
        try {
            const data = await api.updateProgramFlows({ uid: program.uid }, body, { token });
            program.updated = (0, moment_1.default)();
            (0, exports.commitUpdateFeed)({
                type: 'program',
                entity: program,
            })(dispatch);
            return data;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.setProgramData = setProgramData;
const deleteProgram = (program, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        try {
            const isDeleted = await api.deleteProgram({ uid: program.uid }, { token });
            (0, exports.commitDeleteFeed)({
                type: 'program',
                entity: program,
            })(dispatch);
            return isDeleted;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.deleteProgram = deleteProgram;
const getFolderTree = (uid, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        try {
            const folders = await fetchCollection(uid, 'folders', undefined, 1, token);
            const tree = ['/'];
            for (const folder of folders) {
                tree.push(`${folder.path === '/' ? '' : folder.path}/${folder.slug}`);
            }
            return tree.sort();
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.getFolderTree = getFolderTree;
const createFolder = (folder, uid, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        const data = {
            name: folder.name,
            slug: folder.name,
            path: folder.path,
        };
        try {
            const folderFeed = (0, exports.apiToFolderFeedEntity)(await api.createUserFolder({ uid }, data, { token }));
            (0, exports.commitUpdateFeed)({
                type: 'folder',
                entity: folderFeed,
            })(dispatch);
            return folderFeed;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.createFolder = createFolder;
const updateParentFolder = (folder, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        const data = {
            name: folder.name,
            slug: folder.slug,
            path: folder.path,
        };
        try {
            folder = (0, exports.apiToFolderFeedEntity)(await api.updateFolder({ uid: folder.uid }, data, { token }));
            (0, exports.commitSetParentFolderFeed)(folder)(dispatch);
            return folder;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.updateParentFolder = updateParentFolder;
const deleteParentFolder = (folder, token) => {
    return async (dispatch, userDispatch, authDispath) => {
        try {
            const isDeleted = await api.deleteFolder({ uid: folder.uid }, { token });
            (0, exports.commitSetParentFolderFeed)(undefined)(dispatch);
            return isDeleted;
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, user_1.commitLogoutUser)()(userDispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.deleteParentFolder = deleteParentFolder;
exports.FeedContext = react_1.default.createContext({
    feed: defaultState,
    feedDispatch: () => {
        throw new Error('FeedContext not yet initialized.');
    },
    feedRef: {
        current: defaultState,
    },
});
exports.FeedContext.displayName = 'FeedContext';
const FeedProvider = (props) => {
    const [feed, feedDispatch, feedRef] = (0, use_reducer_ref_1.useReducerRef)((state = defaultState, action) => {
        switch (action.type) {
            case FeedActionTypes.COMMIT_CLEAR_FEED:
                return {
                    ...state,
                    items: {},
                };
            case FeedActionTypes.COMMIT_UPDATE_FEED:
                state.items[`${action.item.type}-${action.item.entity.uid}`] = action.item;
                return {
                    ...state,
                };
            case FeedActionTypes.COMMIT_DELETE_FEED:
                delete state.items[`${action.item.type}-${action.item.entity.uid}`];
                return {
                    ...state,
                };
            case FeedActionTypes.COMMIT_SET_PARENT_FOLDER_FEED:
                return {
                    ...state,
                    parentFolder: action.parentFolder,
                };
            case FeedActionTypes.COMMIT_SET_SLUG_FEED:
                return {
                    ...state,
                    slug: action.slug,
                };
            case FeedActionTypes.COMMIT_SET_UID_FEED:
                return {
                    ...state,
                    uid: action.uid,
                };
            default:
                return state;
        }
    }, defaultState);
    return (react_1.default.createElement(exports.FeedContext.Provider, { value: { feed, feedDispatch, feedRef } }, props.children));
};
exports.FeedProvider = FeedProvider;
exports.FeedConsumer = exports.FeedContext.Consumer;
function useFeed() {
    return (0, react_1.useContext)(exports.FeedContext);
}
exports.useFeed = useFeed;
