"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useGraph = exports.GraphConsumer = exports.GraphProvider = exports.GraphContext = exports.commitStopFlow = exports.commitPlayFlow = exports.commitSetFlows = exports.commitUpdateFlow = exports.commitPopFlow = exports.commitPushFlow = exports.GraphActionTypes = void 0;
const react_1 = __importStar(require("react"));
const container_1 = __importDefault(require("../container"));
const use_reducer_ref_1 = require("../hooks/use-reducer-ref");
const services_1 = require("../services");
const container = new container_1.default();
const api = container.get(services_1.Api);
var GraphActionTypes;
(function (GraphActionTypes) {
    GraphActionTypes["COMMIT_PUSH_FLOW"] = "COMMIT_PUSH_FLOW";
    GraphActionTypes["COMMIT_POP_FLOW"] = "COMMIT_POP_FLOW";
    GraphActionTypes["COMMIT_UPDATE_FLOW"] = "COMMIT_UPDATE_FLOW";
    GraphActionTypes["COMMIT_SET_FLOWS"] = "COMMIT_SET_FLOWS";
    GraphActionTypes["COMMIT_PLAY_FLOW"] = "COMMIT_PLAY_FLOW";
    GraphActionTypes["COMMIT_STOP_FLOW"] = "COMMIT_STOP_FLOW";
})(GraphActionTypes = exports.GraphActionTypes || (exports.GraphActionTypes = {}));
;
const defaultState = {
    flows: []
};
const commitPushFlow = (index, flowType) => {
    return async (dispatch) => {
        dispatch({
            type: GraphActionTypes.COMMIT_PUSH_FLOW,
            index,
            flowType,
        });
        return Promise.resolve();
    };
};
exports.commitPushFlow = commitPushFlow;
const commitPopFlow = (index) => {
    return async (dispatch) => {
        dispatch({
            type: GraphActionTypes.COMMIT_POP_FLOW,
            index,
        });
        return Promise.resolve();
    };
};
exports.commitPopFlow = commitPopFlow;
const commitUpdateFlow = (index, data) => {
    return async (dispatch) => {
        dispatch({
            type: GraphActionTypes.COMMIT_UPDATE_FLOW,
            index,
            data,
        });
        return Promise.resolve();
    };
};
exports.commitUpdateFlow = commitUpdateFlow;
const commitSetFlows = (flows) => {
    return async (dispatch) => {
        dispatch({
            type: GraphActionTypes.COMMIT_SET_FLOWS,
            flows,
        });
        return Promise.resolve();
    };
};
exports.commitSetFlows = commitSetFlows;
const commitPlayFlow = (index) => {
    return async (dispatch) => {
        dispatch({
            type: GraphActionTypes.COMMIT_PLAY_FLOW,
            index,
        });
        return Promise.resolve();
    };
};
exports.commitPlayFlow = commitPlayFlow;
const commitStopFlow = (index) => {
    return async (dispatch) => {
        dispatch({
            type: GraphActionTypes.COMMIT_STOP_FLOW,
            index,
        });
        return Promise.resolve();
    };
};
exports.commitStopFlow = commitStopFlow;
exports.GraphContext = (0, react_1.createContext)({
    graph: defaultState,
    graphDispatch: () => {
        throw new Error('GraphContext not yet initialized.');
    },
    graphRef: {
        current: defaultState,
    },
});
exports.GraphContext.displayName = 'GraphContext';
const GraphProvider = (props) => {
    const [graph, graphDispatch, graphRef] = (0, use_reducer_ref_1.useReducerRef)((state = defaultState, action) => {
        switch (action.type) {
            case GraphActionTypes.COMMIT_PUSH_FLOW:
                const pushFlows = state.flows.slice();
                pushFlows.splice(action.index, 0, {
                    type: action.flowType,
                    isPlaying: false,
                });
                return {
                    ...state,
                    ...{ flows: pushFlows }
                };
            case GraphActionTypes.COMMIT_POP_FLOW:
                const popFlows = state.flows.slice();
                popFlows.splice(action.index, 1);
                return {
                    ...state,
                    ...{ flows: popFlows }
                };
            case GraphActionTypes.COMMIT_UPDATE_FLOW:
                return {
                    ...state,
                    ...{ flows: state.flows.map((item, index) => {
                            if (index !== action.index) {
                                return item;
                            }
                            return {
                                ...item,
                                ...{
                                    data: action.data,
                                },
                            };
                        }) }
                };
            case GraphActionTypes.COMMIT_SET_FLOWS:
                return {
                    ...state,
                    ...{ flows: action.flows.slice() }
                };
            case GraphActionTypes.COMMIT_PLAY_FLOW:
                return {
                    ...state,
                    ...{ flows: state.flows.map((item, index) => {
                            if (index !== action.index) {
                                return item;
                            }
                            return {
                                ...item,
                                ...{
                                    isPlaying: true
                                },
                            };
                        }) }
                };
            case GraphActionTypes.COMMIT_STOP_FLOW:
                return {
                    ...state,
                    ...{ flows: state.flows.map((item, index) => {
                            if (index !== action.index) {
                                return item;
                            }
                            return {
                                ...item,
                                ...{
                                    isPlaying: false
                                },
                            };
                        }) }
                };
            default:
                return state;
        }
    }, defaultState);
    return (react_1.default.createElement(exports.GraphContext.Provider, { value: { graph, graphDispatch, graphRef } }, props.children));
};
exports.GraphProvider = GraphProvider;
exports.GraphConsumer = exports.GraphContext.Consumer;
function useGraph() {
    return (0, react_1.useContext)(exports.GraphContext);
}
exports.useGraph = useGraph;
