"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useUser = exports.UserConsumer = exports.UserProvider = exports.UserContext = exports.isGranted = exports.updateSettings = exports.fetchSettings = exports.updateConfig = exports.fetchConfig = exports.commitLogoutUser = exports.commitUpdateSettings = exports.UserActionTypes = void 0;
const react_1 = __importStar(require("react"));
const container_1 = __importDefault(require("../container"));
const services_1 = require("../services");
const uniq_1 = __importDefault(require("lodash/uniq"));
const auth_1 = require("./auth");
const use_reducer_ref_1 = require("../hooks/use-reducer-ref");
const api_type_interfaces_1 = require("../models/api-type-interfaces");
const api_exceptions_1 = require("../models/api-exceptions");
const container = new container_1.default();
const api = container.get(services_1.Api);
var UserActionTypes;
(function (UserActionTypes) {
    UserActionTypes["COMMIT_UPDATE_SETTINGS"] = "COMMIT_UPDATE_SETTINGS";
    UserActionTypes["COMMIT_LOGOUT"] = "COMMIT_LOGOUT";
})(UserActionTypes = exports.UserActionTypes || (exports.UserActionTypes = {}));
const defaultState = {
    uid: undefined,
    apiKey: undefined,
    username: undefined,
    email: undefined,
    firstname: undefined,
    lastname: undefined,
    facebookId: undefined,
    githubId: undefined,
    roles: [],
    links: {
        lead: undefined,
    },
};
const commitUpdateSettings = (user) => {
    return (dispatch) => {
        dispatch({
            type: UserActionTypes.COMMIT_UPDATE_SETTINGS,
            user,
        });
    };
};
exports.commitUpdateSettings = commitUpdateSettings;
const commitLogoutUser = () => {
    return (dispatch, authDispath) => {
        dispatch({
            type: UserActionTypes.COMMIT_LOGOUT,
        });
        (0, auth_1.commitLogout)()(authDispath);
    };
};
exports.commitLogoutUser = commitLogoutUser;
const fetchConfig = (token, uid) => {
    return async (dispatch, authDispath) => {
        try {
            return await api.getAdminConfig({ uid }, { token });
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, exports.commitLogoutUser)()(dispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.fetchConfig = fetchConfig;
const updateConfig = (item, token, uid) => {
    return async (dispatch, authDispath, logsDispatch) => {
        const data = {
            ...item,
        };
        try {
            await api.updateAdminConfig({ uid }, data, { token });
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, exports.commitLogoutUser)()(dispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.updateConfig = updateConfig;
const fetchSettings = (uid, token) => {
    return async (dispatch, authDispath) => {
        try {
            const user = await api.getUserSettings({ uid }, { token });
            (0, exports.commitUpdateSettings)(user)(dispatch);
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, exports.commitLogoutUser)()(dispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.fetchSettings = fetchSettings;
const updateSettings = (item, token) => {
    return async (dispatch, authDispath, logsDispatch) => {
        const data = {
            firstname: item.firstname,
            lastname: item.lastname,
            username: item.username,
            apiKey: item.apiKey,
            facebookId: item.facebookId,
            githubId: item.githubId,
        };
        try {
            if (item.uid) {
                const user = await api.updateUserSettings({ uid: item.uid }, data, { token });
                (0, exports.commitUpdateSettings)(user)(dispatch);
                return user;
            }
        }
        catch (error) {
            if (error instanceof api_exceptions_1.ApiNotAuthorizedException) {
                (0, exports.commitLogoutUser)()(dispatch, authDispath);
            }
            else {
                throw error;
            }
        }
    };
};
exports.updateSettings = updateSettings;
const isGranted = (user, attributes) => {
    if (!Array.isArray(attributes)) {
        attributes = [attributes];
    }
    let roles = [api_type_interfaces_1.ROLE.USER];
    for (let i = 0; i < user.roles.length; i++) {
        const role = user.roles[i];
        if (role === api_type_interfaces_1.ROLE.SUPER_ADMIN) {
            roles.push(api_type_interfaces_1.ROLE.USER);
            roles.push(api_type_interfaces_1.ROLE.SUPER_ADMIN);
        }
        else if (role) {
            roles.push(role);
        }
    }
    roles = (0, uniq_1.default)(roles);
    for (let i = 0; i < attributes.length; i++) {
        const attribute = attributes[i];
        for (let j = 0; j < roles.length; j++) {
            const role = roles[j];
            if (attribute === role) {
                return true;
            }
        }
    }
    return false;
};
exports.isGranted = isGranted;
exports.UserContext = react_1.default.createContext({
    user: defaultState,
    userDispatch: () => {
        throw new Error('UserContext not yet initialized.');
    },
    userRef: {
        current: defaultState,
    },
});
exports.UserContext.displayName = 'UserContext';
const UserProvider = (props) => {
    const [user, userDispatch, userRef] = (0, use_reducer_ref_1.useReducerRef)((state = defaultState, action) => {
        switch (action.type) {
            case UserActionTypes.COMMIT_UPDATE_SETTINGS:
                return {
                    ...state,
                    ...action.user,
                };
            case UserActionTypes.COMMIT_LOGOUT:
                return {
                    ...defaultState,
                };
            default:
                return state;
        }
    }, defaultState);
    return (react_1.default.createElement(exports.UserContext.Provider, { value: { user, userDispatch, userRef } }, props.children));
};
exports.UserProvider = UserProvider;
exports.UserConsumer = exports.UserContext.Consumer;
function useUser() {
    return (0, react_1.useContext)(exports.UserContext);
}
exports.useUser = useUser;
