"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.matchRoute = exports.pathTo = void 0;
const home_1 = __importDefault(require("./views/home"));
const login_1 = __importDefault(require("./views/login/login"));
const facebook_1 = __importDefault(require("./views/login/facebook"));
const github_1 = __importDefault(require("./views/login/github"));
const register_1 = __importDefault(require("./views/login/register"));
const flows_1 = __importDefault(require("./views/flows"));
const contact_1 = __importDefault(require("./views/contact"));
const doc_1 = __importDefault(require("./views/doc/doc"));
const newsletter_1 = __importDefault(require("./views/newsletter"));
const notifications_1 = __importDefault(require("./views/notifications"));
const changelog_1 = __importDefault(require("./views/changelog"));
const contributor_1 = __importDefault(require("./views/blog/contributor"));
const tag_1 = __importDefault(require("./views/blog/tag"));
const tags_1 = __importDefault(require("./views/blog/tags"));
const article_1 = __importDefault(require("./views/blog/article"));
const blog_1 = __importDefault(require("./views/blog/blog"));
const card_1 = __importDefault(require("./views/library/card"));
const library_1 = __importDefault(require("./views/library/library"));
const settings_1 = __importDefault(require("./views/settings"));
const admin_1 = __importDefault(require("./views/admin"));
const feed_1 = __importDefault(require("./views/feed/feed"));
const path_to_regexp_1 = require("path-to-regexp");
const helpers_1 = require("./helpers");
const services_1 = require("./services");
const container_1 = __importDefault(require("./container"));
const api_type_interfaces_1 = require("./models/api-type-interfaces");
const container = new container_1.default();
const path = container.get(services_1.Path);
const routes = {
    home: {
        path: '/',
        exact: true,
        component: home_1.default,
    },
    login: {
        path: '/login',
        exact: true,
        component: login_1.default,
    },
    facebookLogin: {
        path: '/login/facebook',
        exact: true,
        component: facebook_1.default,
    },
    githubLogin: {
        path: '/login/github',
        exact: true,
        component: github_1.default,
    },
    register: {
        path: '/register',
        exact: true,
        component: register_1.default,
    },
    flows: {
        path: '/flows',
        component: flows_1.default,
    },
    contact: {
        path: '/contact',
        component: contact_1.default,
    },
    doc: {
        path: '/docs/:slug?',
        component: doc_1.default,
    },
    newsletter: {
        path: '/newsletters/:slug',
        component: newsletter_1.default,
    },
    notificationUnsubscribe: {
        path: '/notifications/unsubscribe',
        component: notifications_1.default,
    },
    notificationManage: {
        path: '/notifications/manage',
        component: notifications_1.default,
    },
    changelog: {
        path: '/changelog',
        component: changelog_1.default,
    },
    contributor: {
        path: '/blog/contributors/:slug',
        component: contributor_1.default,
    },
    tag: {
        path: '/blog/tags/:tag',
        component: tag_1.default,
    },
    tags: {
        path: '/blog/tags',
        component: tags_1.default,
    },
    article: {
        path: '/blog/:slug',
        component: article_1.default,
    },
    blog: {
        path: '/blog',
        component: blog_1.default,
    },
    card: {
        path: '/library/:slug',
        component: card_1.default,
    },
    library: {
        path: '/library',
        component: library_1.default,
    },
    settings: {
        path: '/settings',
        component: (0, helpers_1.requireAuthentication)(settings_1.default),
    },
    admin: {
        path: '/admin',
        component: (0, helpers_1.requireAuthentication)(admin_1.default, api_type_interfaces_1.ROLE.SUPER_ADMIN),
    },
    feed: {
        path: '/me/feed/:slug1?/:slug2?/:slug3?/:slug4?/:slug5?',
        component: (0, helpers_1.requireAuthentication)(feed_1.default),
    },
    userFeed: {
        path: '/:uid/feed/:slug1?/:slug2?/:slug3?/:slug4?/:slug5?',
        component: feed_1.default,
    },
};
const pathTo = (route, params = {}) => {
    if (!(route in routes)) {
        throw new Error(`There is no such view as ${route}`);
    }
    return (0, path_to_regexp_1.compile)(routes[route].path)(params);
};
exports.pathTo = pathTo;
const matchRoute = (pathname) => {
    const keys = Object.keys(routes);
    let match = null;
    let route = null;
    for (let i = 0; i < keys.length; i++) {
        route = keys[i];
        match = path.matchPath(pathname, {
            path: routes[route].path,
            exact: routes[route].exact,
        });
        if (match) {
            return {
                route: route,
                match: match,
            };
        }
    }
    return null;
};
exports.matchRoute = matchRoute;
exports.default = routes;
