"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typedi_1 = require("typedi");
const axios_1 = __importDefault(require("axios"));
const env_1 = __importDefault(require("./env"));
const api_exceptions_1 = require("../models/api-exceptions");
const axios_2 = __importDefault(require("axios"));
const api_exceptions_2 = require("../models/api-exceptions");
const api_exceptions_3 = require("../models/api-exceptions");
let Api = class Api {
    env;
    getBaseUrl() {
        return `${this.env.get('apiUrl')}/api`;
    }
    getQuery(query) {
        if (query === undefined) {
            return '';
        }
        const paths = Object.keys(query)
            .filter((key) => {
            return query[key] !== undefined;
        })
            .map((key) => {
            return `${key}=${query[key]}`;
        });
        return paths.length > 0 ? `?${paths.join('&')}` : '';
    }
    getOptions(options) {
        return options?.token
            ? {
                headers: {
                    'Uniflow-Authorization': `Bearer ${options.token}`,
                },
            }
            : {};
    }
    handleApiError(error) {
        if (axios_2.default.isAxiosError(error)) {
            const axiosError = error;
            if (axiosError.response) {
                if (axiosError.response.status === 400) {
                    return new api_exceptions_3.ApiNotFoundException();
                }
                else if (axiosError.response.status === 401) {
                    return new api_exceptions_2.ApiNotAuthorizedException();
                }
                else if (axiosError.response.status === 422) {
                    const errors = {};
                    if (axiosError.response.data.validation) {
                        for (const item of axiosError.response.data.validation) {
                            errors[item.key] = errors[item.key] || [];
                            for (const message of item.messages) {
                                errors[item.key]?.push(message);
                            }
                        }
                    }
                    return new api_exceptions_1.ApiValidateException(errors);
                }
            }
        }
        return error;
    }
    async login(body) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/login`, body);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async loginFacebook(body, options) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/login-facebook`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async loginGithub(body, options) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/login-github`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async contact(body) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/contacts`, body);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async updateFolder(path, body, options) {
        try {
            const response = await axios_1.default.put(`${this.getBaseUrl()}/folders/${path.uid}`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async deleteFolder(path, options) {
        try {
            const response = await axios_1.default.delete(`${this.getBaseUrl()}/folders/${path.uid}`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async createLead(body) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/leads`, body);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getLead(path) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/leads/${path.uid}`);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async updateLead(path, body) {
        try {
            const response = await axios_1.default.put(`${this.getBaseUrl()}/leads/${path.uid}`, body);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getPrograms(query) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/programs${this.getQuery(query)}`);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async updateProgram(path, body, options) {
        try {
            const response = await axios_1.default.put(`${this.getBaseUrl()}/programs/${path.uid}`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async deleteProgram(path, options) {
        try {
            const response = await axios_1.default.delete(`${this.getBaseUrl()}/programs/${path.uid}`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getProgramFlows(path, options) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/programs/${path.uid}/flows`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async updateProgramFlows(path, body, options) {
        try {
            const response = await axios_1.default.put(`${this.getBaseUrl()}/programs/${path.uid}/flows`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async createUser(body) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/users`, body);
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getUserSettings(path, options) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/users/${path.uid}/settings`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async updateUserSettings(path, body, options) {
        try {
            const response = await axios_1.default.put(`${this.getBaseUrl()}/users/${path.uid}/settings`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getAdminConfig(path, options) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/users/${path.uid}/admin-config`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async updateAdminConfig(path, body, options) {
        try {
            const response = await axios_1.default.put(`${this.getBaseUrl()}/users/${path.uid}/admin-config`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getUserFolders(path, query, options) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/users/${path.uid}/folders${this.getQuery(query)}`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async createUserFolder(path, body, options) {
        try {
            const reponse = await axios_1.default.post(`${this.getBaseUrl()}/users/${path.uid}/folders`, body, this.getOptions(options));
            return reponse.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async getUserPrograms(path, query, options) {
        try {
            const response = await axios_1.default.get(`${this.getBaseUrl()}/users/${path.uid}/programs${this.getQuery(query)}`, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
    async createUserProgram(path, body, options) {
        try {
            const response = await axios_1.default.post(`${this.getBaseUrl()}/users/${path.uid}/programs`, body, this.getOptions(options));
            return response.data;
        }
        catch (error) {
            throw this.handleApiError(error);
        }
    }
};
__decorate([
    (0, typedi_1.Inject)(() => env_1.default),
    __metadata("design:type", env_1.default)
], Api.prototype, "env", void 0);
Api = __decorate([
    (0, typedi_1.Service)()
], Api);
exports.default = Api;
