"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.query = void 0;
const react_1 = __importDefault(require("react"));
const routes_1 = require("../../routes");
const gatsby_1 = require("gatsby");
const gatsby_plugin_image_1 = require("gatsby-plugin-image");
const free_solid_svg_icons_1 = require("@fortawesome/free-solid-svg-icons");
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const ArticleItem = ({ article }) => (react_1.default.createElement("div", { className: "card card-link" },
    react_1.default.createElement(gatsby_1.Link, { className: "btn btn-default", to: (0, routes_1.pathTo)('article', { slug: article.fields.slug }) }, "Read More"),
    (article.frontmatter.cover.childImageSharp && (react_1.default.createElement(gatsby_plugin_image_1.GatsbyImage, { image: article.frontmatter.cover.childImageSharp.gatsbyImageData, className: "card-img-top", style: { maxHeight: '50vh' }, alt: "Cover" }))) ||
        (article.frontmatter.cover.extension === 'svg' && (react_1.default.createElement("img", { className: "card-img-top", src: article.frontmatter.cover.publicURL, style: { maxHeight: '50vh' }, alt: "Cover" }))),
    react_1.default.createElement("div", { className: "card-body" },
        react_1.default.createElement("h4", { className: "card-title text-center" }, article.frontmatter.title),
        react_1.default.createElement("p", { className: "card-text text-center" }, article.excerpt),
        react_1.default.createElement("div", { className: "row pt-1" },
            react_1.default.createElement("div", { className: "col-auto" },
                react_1.default.createElement("small", null,
                    "Posted ",
                    article.frontmatter.date,
                    " by",
                    react_1.default.createElement(gatsby_plugin_image_1.GatsbyImage, { image: article.frontmatter.author.image.childImageSharp.gatsbyImageData, alt: article.frontmatter.author.name, className: "rounded-circle mx-2", style: { verticalAlign: 'middle', display: 'inline-block' } }),
                    react_1.default.createElement(gatsby_1.Link, { to: (0, routes_1.pathTo)('contributor', {
                            slug: article.frontmatter.author.fields.slug,
                        }) }, article.frontmatter.author.name),
                    react_1.default.createElement("span", null, " - "),
                    react_1.default.createElement("i", null,
                        article.timeToRead,
                        " min read"))),
            react_1.default.createElement("div", { className: "col-auto ms-auto" }, article.frontmatter.tags.map((tag, k) => (react_1.default.createElement(gatsby_1.Link, { to: (0, routes_1.pathTo)('tag', { tag: tag }), key: k, className: "btn btn-sm btn-primary" },
                react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faTag }),
                " ",
                tag))))))));
exports.query = (0, gatsby_1.graphql) `
  fragment ArticleItemFragment on Mdx {
    fields {
      slug
    }
    excerpt
    frontmatter {
      title
      author {
        fields {
          slug
        }
        name
        image {
          childImageSharp {
            gatsbyImageData(width: 36, height: 36, layout: FIXED)
          }
        }
      }
      cover {
        childImageSharp {
          gatsbyImageData(layout: FULL_WIDTH)
        }
        extension
        publicURL
      }
      tags
      date(formatString: "MMMM Do YYYY")
    }
    timeToRead
  }
`;
exports.default = ArticleItem;
