"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const gatsby_1 = require("gatsby");
const gatsby_2 = require("gatsby");
const feed_1 = require("../../contexts/feed");
const logs_1 = require("../../contexts/logs");
const free_solid_svg_icons_1 = require("@fortawesome/free-solid-svg-icons");
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const contexts_1 = require("../../contexts");
const react_2 = require("react");
const Navigation = (props) => {
    const [search, setSearch] = (0, react_1.useState)('');
    const [isCollapsed, setIsCollapsed] = (0, react_1.useState)(true);
    const { auth } = (0, contexts_1.useAuth)();
    const { logsDispatch } = (0, logs_1.useLogs)();
    const { user, userDispatch } = (0, contexts_1.useUser)();
    const { feed, feedDispatch } = (0, feed_1.useFeed)();
    const { authDispatch } = (0, contexts_1.useAuth)();
    (0, react_2.useEffect)(() => {
        setIsCollapsed(props.type === 'program');
    }, [props.type]);
    const onSearch = (event) => {
        setSearch(event.target.value);
    };
    const onToggle = (event) => {
        setIsCollapsed(!isCollapsed);
    };
    const onCreateFolder = async (event) => {
        event.preventDefault();
        const folderPath = feed.parentFolder
            ? `${feed.parentFolder.path}/${feed.parentFolder.slug}`
            : '/';
        try {
            if (auth.uid && auth.token) {
                const item = await (0, feed_1.createFolder)({
                    name: search,
                    path: folderPath,
                }, auth.uid, auth.token)(feedDispatch, userDispatch, authDispatch);
                (0, feed_1.commitSetSlugFeed)(null)(feedDispatch);
                if (item) {
                    (0, gatsby_2.navigate)((0, feed_1.toFeedPath)(item, user));
                }
            }
        }
        catch (error) {
            if (error instanceof Error) {
                (0, logs_1.commitAddLog)(error.message)(logsDispatch);
            }
        }
    };
    const onSubmit = async (event) => {
        event.preventDefault();
        const folderPath = feed.parentFolder
            ? `${feed.parentFolder.path}/${feed.parentFolder.slug}`
            : '/';
        try {
            if (auth.uid && auth.token) {
                const item = await (0, feed_1.createProgram)({
                    name: search,
                    clients: ['uniflow'],
                    tags: [],
                    path: folderPath,
                }, auth.uid, auth.token)(feedDispatch, userDispatch, authDispatch);
                if (item) {
                    (0, gatsby_2.navigate)((0, feed_1.toFeedPath)(item, user));
                }
            }
        }
        catch (error) {
            if (error instanceof Error) {
                (0, logs_1.commitAddLog)(error.message)(logsDispatch);
            }
        }
    };
    const currentItem = (0, feed_1.getFeedItem)(feed);
    const folderActiveClass = () => {
        return feed.parentFolder && feed.parentFolder.slug === feed.slug ? 'active' : undefined;
    };
    const itemActiveClass = (item) => {
        return currentItem && currentItem.entity.slug === item.entity.slug ? 'active' : undefined;
    };
    return (react_1.default.createElement("div", { className: "sidebar" },
        react_1.default.createElement("form", { className: "sidebar-search d-flex align-items-center", role: "search", onSubmit: onSubmit },
            react_1.default.createElement("div", { className: "input-group" },
                react_1.default.createElement("input", { type: "search", className: "form-control ds-input", placeholder: "Search...", "aria-label": "Search for...", value: search, onChange: onSearch }),
                search && (react_1.default.createElement("button", { className: "input-group-text", type: "button", onClick: onCreateFolder },
                    react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faFolder })))),
            react_1.default.createElement("button", { className: "btn d-sm-none py-0 px-0 ms-3 isCollapsedd", type: "button", onClick: onToggle }, (isCollapsed && (react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", className: "bi bi-expand", fill: "currentColor", viewBox: "0 0 16 16" },
                react_1.default.createElement("title", null, "Expand"),
                react_1.default.createElement("use", { xlinkHref: "#nav-expand" })))) || (react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", className: "bi bi-expand", fill: "currentColor", viewBox: "0 0 16 16" },
                react_1.default.createElement("title", null, "Collapse"),
                react_1.default.createElement("use", { xlinkHref: "#nav-collapse" }))))),
        react_1.default.createElement("nav", { className: `sidebar-nav${isCollapsed ? ' d-none d-sm-block' : ''}` },
            react_1.default.createElement("div", { className: "sidebar-section" },
                react_1.default.createElement("ul", { className: "sidebar-items" },
                    feed.parentFolder && [
                        react_1.default.createElement("li", { key: 'back' },
                            react_1.default.createElement("span", { className: "link" },
                                react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faArrowLeft }),
                                " Back"),
                            react_1.default.createElement(gatsby_1.Link, { to: (0, feed_1.toFeedPath)(feed.parentFolder, user, true) }, "Back")),
                        react_1.default.createElement("li", { className: folderActiveClass(), key: 'folder' },
                            react_1.default.createElement("span", { className: "link" }, "."),
                            react_1.default.createElement(gatsby_1.Link, { to: (0, feed_1.toFeedPath)(feed.parentFolder, user) }, ".")),
                    ],
                    (0, feed_1.getOrderedFeed)(feed, search).map((item, i) => (react_1.default.createElement("li", { className: itemActiveClass(item), key: i },
                        react_1.default.createElement("span", { className: "link" },
                            item.type === 'folder' && (react_1.default.createElement(react_1.default.Fragment, null,
                                react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faFolder }),
                                " ",
                                item.entity.name,
                                ' ')),
                            item.type === 'program' && react_1.default.createElement(react_1.default.Fragment, null,
                                item.entity.name,
                                " "),
                            item.type === 'program' &&
                                'tags' in item.entity &&
                                item.entity.tags.map((tag, j) => (react_1.default.createElement("span", { key: j, className: "badge badge-light mr-1" }, tag)))),
                        react_1.default.createElement(gatsby_1.Link, { to: (0, feed_1.toFeedPath)(item.entity, user) }, item.entity.name)))))))));
};
exports.default = Navigation;
