"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const gatsby_1 = require("gatsby");
const debounce_1 = __importDefault(require("lodash/debounce"));
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const free_solid_svg_icons_1 = require("@fortawesome/free-solid-svg-icons");
const free_regular_svg_icons_1 = require("@fortawesome/free-regular-svg-icons");
const components_1 = require("../../components");
const runner_1 = __importDefault(require("../../models/runner"));
const graph_1 = require("../../contexts/graph");
const feed_1 = require("../../contexts/feed");
const logs_1 = require("../../contexts/logs");
const container_1 = __importDefault(require("../../container"));
const services_1 = require("../../services");
const contexts_1 = require("../../contexts");
const api_exceptions_1 = require("../../models/api-exceptions");
const form_input_1 = __importStar(require("../../components/form-input"));
const router_1 = require("@reach/router");
const alert_1 = __importStar(require("../../components/alert"));
const react_2 = require("react");
const interfaces_1 = require("../../models/interfaces");
const container = new container_1.default();
const ui = container.get(services_1.UI);
const Program = (props) => {
    const [folderTreeEdit, setFolderTreeEdit] = (0, react_1.useState)(false);
    const [folderTree, setFolderTree] = (0, react_1.useState)([]);
    const [errors, setErrors] = (0, react_1.useState)({});
    const { logsDispatch } = (0, logs_1.useLogs)();
    const { auth, authDispatch } = (0, contexts_1.useAuth)();
    const { user, userDispatch } = (0, contexts_1.useUser)();
    const { feed, feedDispatch, feedRef } = (0, feed_1.useFeed)();
    const { graph, graphDispatch, graphRef } = (0, contexts_1.useGraph)();
    const [fetchedFlows, setFetchedFlows] = (0, react_1.useState)([]);
    const flowsRef = (0, react_2.useRef)(null);
    const location = (0, router_1.useLocation)();
    const { program, allFlows } = props;
    const clients = {
        [interfaces_1.ClientType.UNIFLOW]: 'Uniflow',
        [interfaces_1.ClientType.NODE]: 'Node',
        [interfaces_1.ClientType.VSCODE]: 'VSCode',
    };
    const getProgramRef = () => {
        return (0, feed_1.getFeedItem)(feedRef.current).entity;
    };
    const getFlows = (program) => {
        const flowLabels = [];
        const keys = Object.keys(allFlows);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            const canPushFlow = program.clients.reduce((bool, client) => {
                return bool && allFlows[key].clients.indexOf(client) !== -1;
            }, program.clients.length > 0);
            if (canPushFlow) {
                flowLabels.push({
                    key: key,
                    label: allFlows[key].tags.join(' - ') + ' : ' + allFlows[key].name,
                });
            }
        }
        flowLabels.sort(function (flow1, flow2) {
            const x = flow1.label;
            const y = flow2.label;
            return x < y ? -1 : x > y ? 1 : 0;
        });
        return flowLabels;
    };
    const programFlows = getFlows(program);
    const tags = (0, feed_1.getTags)(feed);
    (0, react_1.useEffect)(() => {
        setFolderTreeEdit(false);
        setFolderTree([props.program.path]);
        onFetchFlowData();
        return () => {
            onFetchFlowData.cancel();
            onUpdate.cancel();
            onUpdateFlowData.cancel();
        };
    }, [props.program.uid]);
    const onPlay = async (index) => {
        const runner = new runner_1.default();
        await runner.run(graph.flows.slice(0, index === undefined ? graph.flows.length : index + 1), flowsRef, graphDispatch);
    };
    const onPushFlow = (index, flowType) => {
        (0, graph_1.commitPushFlow)(index, flowType)(graphDispatch);
        onUpdateFlowData();
    };
    const onPopFlow = (index) => {
        (0, graph_1.commitPopFlow)(index)(graphDispatch);
        onUpdateFlowData();
    };
    const onUpdateFlow = (index, data) => {
        (0, graph_1.commitUpdateFlow)(index, data)(graphDispatch);
        onUpdateFlowData();
    };
    (0, react_1.useEffect)(() => {
        for (let index = 0; index < graph.flows.length; index++) {
            const flow = graph.flows[index];
            if (flow.data === undefined && flowsRef.current) {
                const data = flowsRef.current.onDeserialize(index, fetchedFlows[index]?.data);
                onUpdateFlow(index, data);
            }
        }
    }, [graph.flows.length]);
    const onChangeName = async (name) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ name },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onChangeSlug = (slug) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ slug },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onChangePath = async (path) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ path },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onChangeClients = async (clients) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ clients },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onChangeTags = async (tags) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ tags },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onChangeDescription = async (description) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ description },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onChangePublic = async (isPublic) => {
        (0, feed_1.commitUpdateFeed)({
            type: 'program',
            entity: {
                ...props.program,
                ...{ isPublic },
            },
        })(feedDispatch);
        onUpdate();
    };
    const onSerializeFlowsData = (flows) => {
        const data = [];
        for (let index = 0; index < flows.length; index++) {
            const flow = flows[index];
            data.push({
                flow: flow.type,
                data: flowsRef.current?.onSerialize(index),
            });
        }
        return JSON.stringify(data);
    };
    const onDeserializeFlowsData = (raw) => {
        const data = JSON.parse(raw);
        const graph = [];
        for (let index = 0; index < data.length; index++) {
            graph.push({
                type: data[index].flow,
                data: data[index].data,
            });
        }
        return graph;
    };
    const onFetchFlowData = (0, react_1.useMemo)(() => (0, debounce_1.default)(async () => {
        const programRef = getProgramRef();
        (0, graph_1.commitSetFlows)([])(graphDispatch);
        let data = programRef.data;
        if (!data) {
            data = await (0, feed_1.getProgramData)(programRef, auth.token)(feedDispatch, userDispatch, authDispatch) || null;
        }
        if (data) {
            programRef.data = data;
            const graphData = onDeserializeFlowsData(data);
            setFetchedFlows(graphData);
            for (let index = 0; index < graphData.length; index++) {
                onPushFlow(index, graphData[index].type);
            }
        }
    }, 1000), [props.program.uid]);
    const onUpdateFlowData = (0, react_1.useMemo)(() => (0, debounce_1.default)(async () => {
        const programRef = getProgramRef();
        const data = onSerializeFlowsData(graphRef.current.flows);
        if (auth.token &&
            (feedRef.current.uid === 'me' || user.uid === feedRef.current.uid) &&
            programRef.data !== data) {
            programRef.data = data;
            try {
                await (0, feed_1.setProgramData)(programRef, auth.token)(feedDispatch, userDispatch, authDispatch);
            }
            catch (error) {
                if (error instanceof Error) {
                    (0, logs_1.commitAddLog)(error.message)(logsDispatch);
                }
            }
        }
    }, 1000), [props.program.uid]);
    const onUpdate = (0, react_1.useMemo)(() => (0, debounce_1.default)(async () => {
        if (auth.token) {
            try {
                setErrors({});
                await (0, feed_1.updateProgram)(getProgramRef(), auth.token)(feedDispatch, userDispatch, authDispatch);
                const path = (0, feed_1.toFeedPath)(getProgramRef(), user);
                if (getProgramRef().slug && path !== location.pathname) {
                    (0, gatsby_1.navigate)(path);
                }
            }
            catch (error) {
                if (error instanceof api_exceptions_1.ApiValidateException) {
                    setErrors({ ...error.errors });
                }
                else if (error instanceof Error) {
                    setErrors({ form: [error.message] });
                }
            }
        }
    }, 1000), [props.program.uid]);
    const onDuplicate = async (event) => {
        event.preventDefault();
        if (auth.token && auth.uid) {
            program.name += ' Copy';
            try {
                const item = (0, feed_1.createProgram)(program, auth.uid, auth.token)(feedDispatch, userDispatch, authDispatch);
                Object.assign(program, item);
                await (0, feed_1.setProgramData)(program, auth.token)(feedDispatch, userDispatch, authDispatch);
                (0, gatsby_1.navigate)((0, feed_1.toFeedPath)(program, user));
            }
            catch (error) {
                if (error instanceof Error) {
                    (0, logs_1.commitAddLog)(error.message)(logsDispatch);
                }
            }
        }
    };
    const onDelete = async (event) => {
        event.preventDefault();
        if (auth.token) {
            await (0, feed_1.deleteProgram)(props.program, auth.token)(feedDispatch, userDispatch, authDispatch);
            (0, gatsby_1.navigate)((0, feed_1.toFeedPath)(props.program, user, true));
        }
    };
    const onFolderEdit = async (event) => {
        event.preventDefault();
        if (feed.uid) {
            const folderTree = await (0, feed_1.getFolderTree)(feed.uid, auth.token)(feedDispatch, userDispatch, authDispatch) || [];
            setFolderTreeEdit(true);
            setFolderTree(folderTree);
        }
    };
    const getNodeClipboard = () => {
        if (user.apiKey) {
            return `node -e "$(curl -s https://uniflow.io/assets/node.js)" - --api-key=${user.apiKey} ${program.slug}`;
        }
        return `node -e "$(curl -s https://uniflow.io/assets/node.js)" - --api-key={your-api-key} ${program.slug}`;
    };
    const onCopyNodeUsage = (event) => {
        event.preventDefault();
        const clipboard = getNodeClipboard();
        ui.copyTextToClipboard(clipboard);
    };
    return (react_1.default.createElement("section", { className: "section section-with-sidebar col" },
        react_1.default.createElement("div", { className: "row" },
            react_1.default.createElement("div", { className: "col" },
                react_1.default.createElement("h3", null, "Infos")),
            react_1.default.createElement("div", { className: "d-block col-auto" },
                react_1.default.createElement("div", { className: "btn-toolbar", role: "toolbar", "aria-label": "flow actions" },
                    react_1.default.createElement("div", { className: "btn-group-sm", role: "group" },
                        react_1.default.createElement("button", { type: "button", className: "btn text-secondary", onClick: onDuplicate },
                            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faClone })),
                        react_1.default.createElement("button", { type: "button", className: "btn text-secondary", onClick: onDelete },
                            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faTimes })))))),
        react_1.default.createElement("form", { className: "form-sm-horizontal" },
            errors.form &&
                errors.form.map((message, i) => (react_1.default.createElement(alert_1.default, { key: i, type: alert_1.AlertType.DANGER }, message))),
            react_1.default.createElement(form_input_1.default, { id: "program-name", type: form_input_1.FormInputType.TEXT, label: "Name", value: program.name, errors: errors.name, onChange: onChangeName }),
            react_1.default.createElement(form_input_1.default, { id: "program-slug", type: form_input_1.FormInputType.TEXT, label: "Slug", value: program.slug, errors: errors.slug, onChange: onChangeSlug }),
            react_1.default.createElement("div", { className: "row mb-3" },
                react_1.default.createElement("label", { htmlFor: "program-path", className: "col-sm-2 col-form-label" }, "Path"),
                react_1.default.createElement("div", { className: "col-sm-10" }, (folderTreeEdit && (react_1.default.createElement(components_1.Select, { value: program.path, onChange: onChangePath, className: "form-control", id: "program-path", options: folderTree.map((value) => {
                        return { value: value, label: value };
                    }) }))) || (react_1.default.createElement("div", null,
                    react_1.default.createElement("button", { type: "button", className: "btn btn-secondary", onClick: onFolderEdit },
                        react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faEdit })),
                    ' ',
                    program.path)))),
            react_1.default.createElement(form_input_1.default, { id: "program-clients", type: form_input_1.FormInputType.SELECT, label: "Clients", value: program.clients, errors: errors.clients, onChange: onChangeClients, multiple: true, options: Object.keys(clients).map((value) => {
                    return { value: value, label: clients[value] };
                }) }),
            react_1.default.createElement(form_input_1.default, { id: "program-tags", type: form_input_1.FormInputType.SELECT, label: "Tags", value: program.tags, errors: errors.tags, onChange: onChangeTags, multiple: true, options: tags.map((tag) => {
                    return { value: tag, label: tag };
                }) }),
            react_1.default.createElement(form_input_1.default, { id: "program-public", type: form_input_1.FormInputType.CHECKBOX, label: "Public", value: program.isPublic, errors: errors.isPublic, onChange: onChangePublic }),
            react_1.default.createElement(form_input_1.default, { id: "program-description", type: form_input_1.FormInputType.EDITOR, label: "Description", value: program.description, errors: errors.description, onChange: onChangeDescription })),
        program.clients.map((client) => {
            if (client === 'uniflow') {
                return (react_1.default.createElement("div", { key: `client-${client}`, className: "row mb-3" },
                    react_1.default.createElement("div", { className: "col-sm-10 offset-sm-2" },
                        react_1.default.createElement("button", { className: "btn btn-primary", onClick: (event) => {
                                event.preventDefault();
                                onPlay();
                            } },
                            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_solid_svg_icons_1.faPlay }),
                            " Play"))));
            }
            else if (client === 'node') {
                const clipboard = getNodeClipboard();
                return (react_1.default.createElement("div", { key: `client-${client}`, className: "row mb-3" },
                    react_1.default.createElement("label", { htmlFor: "program-node-api-key", className: "col-sm-2 col-form-label" }, "Node usage"),
                    react_1.default.createElement("div", { className: "col-sm-10" },
                        react_1.default.createElement("div", { className: "input-group" },
                            react_1.default.createElement("button", { type: "button", className: "input-group-text", onClick: onCopyNodeUsage },
                                react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_regular_svg_icons_1.faClipboard })),
                            react_1.default.createElement("input", { type: "text", className: "form-control", id: "program-node-api-key", value: clipboard || '', readOnly: true, placeholder: "api key" })))));
            }
            return null;
        }),
        react_1.default.createElement("hr", null),
        react_1.default.createElement(components_1.Flows, { ref: flowsRef, clients: program.clients, graph: graph, programFlows: programFlows, onPush: onPushFlow, onPop: onPopFlow, onUpdate: onUpdateFlow, onPlay: onPlay })));
};
exports.default = Program;
