"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const free_brands_svg_icons_1 = require("@fortawesome/free-brands-svg-icons");
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const free_regular_svg_icons_1 = require("@fortawesome/free-regular-svg-icons");
const auth_1 = require("../contexts/auth");
const user_1 = require("../contexts/user");
const container_1 = __importDefault(require("../container"));
const services_1 = require("../services");
const react_2 = require("react");
const react_3 = require("react");
const contexts_1 = require("../contexts");
const use_state_ref_1 = require("../hooks/use-state-ref");
const api_exceptions_1 = require("../models/api-exceptions");
const alert_1 = __importStar(require("../components/alert"));
const form_input_1 = __importStar(require("../components/form-input"));
const container = new container_1.default();
const ui = container.get(services_1.UI);
const env = container.get(services_1.Env);
const api = container.get(services_1.Api);
const Settings = () => {
    const [userSettings, setUserSettings, userSettingsRef] = (0, use_state_ref_1.useStateRef)({
        apiKey: undefined,
        username: undefined,
        email: undefined,
        firstname: undefined,
        lastname: undefined,
        facebookId: undefined,
        githubId: undefined,
        links: {
            lead: undefined,
        },
    });
    const [leadSettings, setLeadSettings, leadRef] = (0, use_state_ref_1.useStateRef)({
        optinNewsletter: false,
        optinBlog: false,
        optinGithub: false,
        githubUsername: null,
    });
    const [errors, setErrors] = (0, react_2.useState)({});
    const [state, setState] = (0, react_2.useState)('form');
    const { auth, authDispatch } = (0, contexts_1.useAuth)();
    const { user, userDispatch } = (0, user_1.useUser)();
    const { logsDispatch } = (0, contexts_1.useLogs)();
    (0, react_3.useEffect)(() => {
        (async () => {
            setUserSettings({ ...user });
            if (user.links.lead) {
                const data = await api.getLead({ uid: user.links.lead });
                setLeadSettings({
                    ...leadSettings,
                    ...{
                        optinNewsletter: data.optinNewsletter,
                        optinBlog: data.optinBlog,
                        optinGithub: data.optinGithub,
                        githubUsername: data.githubUsername,
                    },
                });
            }
        })();
    }, [user]);
    const onUpdateFirstname = (value) => {
        setUserSettings({ ...userSettings, ...{ firstname: value } });
    };
    const onUpdateLastname = (value) => {
        setUserSettings({ ...userSettings, ...{ lastname: value } });
    };
    const onUpdateUsername = (value) => {
        setUserSettings({ ...userSettings, ...{ username: value } });
    };
    const onUpdateApiKey = (value) => {
        setUserSettings({ ...userSettings, ...{ apiKey: value } });
    };
    const onRevokeFacebook = async (event) => {
        event.preventDefault();
        setUserSettings({ ...userSettings, ...{ facebookId: undefined } });
        await onUpdate();
    };
    const onRevokeGithub = async (event) => {
        event.preventDefault();
        setUserSettings({ ...userSettings, ...{ githubId: undefined } });
        await onUpdate();
    };
    const onChangeOptinNewsletter = (value) => {
        setLeadSettings({ ...leadSettings, ...{ optinNewsletter: value } });
    };
    const onChangeOptinBlog = (value) => {
        setLeadSettings({ ...leadSettings, ...{ optinBlog: value } });
    };
    const onChangeOptinGithub = (value) => {
        setLeadSettings({ ...leadSettings, ...{ optinGithub: value } });
    };
    const onUpdate = async () => {
        setState('form-submit');
        try {
            setErrors({});
            if (userSettingsRef.current.links.lead) {
                await api.updateLead({ uid: userSettingsRef.current.links.lead }, {
                    optinNewsletter: leadRef.current.optinNewsletter,
                    optinBlog: leadRef.current.optinBlog,
                    optinGithub: leadRef.current.optinGithub,
                });
            }
            else if (userSettingsRef.current.email &&
                (leadRef.current.optinNewsletter || leadRef.current.optinBlog)) {
                await api.createLead({
                    email: userSettingsRef.current.email,
                    optinNewsletter: leadRef.current.optinNewsletter,
                    optinBlog: leadRef.current.optinBlog,
                });
            }
            if (auth.token) {
                await (0, user_1.updateSettings)(userSettingsRef.current, auth.token)(userDispatch, authDispatch, logsDispatch);
            }
            setState('form-success');
        }
        catch (error) {
            setState('form');
            if (error instanceof api_exceptions_1.ApiValidateException) {
                setErrors({ ...error.errors });
            }
            else if (error instanceof Error) {
                setErrors({ form: [error.message] });
            }
        }
    };
    const onSubmit = async (event) => {
        event.preventDefault();
        await onUpdate();
    };
    const onGenerateKey = async (event) => {
        event.preventDefault();
        const chars = 'abcdefghijklmnopqrstuvwxyz0123456789';
        let apiKey = '';
        for (let i = 0; i < 32; i++) {
            apiKey += chars.charAt(Math.floor(Math.random() * chars.length));
        }
        setUserSettings({ ...userSettings, ...{ apiKey: apiKey } });
        await onUpdate();
    };
    const getClipboard = () => {
        if (userSettings.apiKey) {
            return `node -e "$(curl -s https://uniflow.io/assets/node.js)" - --api-key=${userSettings.apiKey}`;
        }
        return undefined;
    };
    const onCopyApiUsage = (event) => {
        const clipboard = getClipboard();
        ui.copyTextToClipboard(clipboard || '');
    };
    const clipboard = getClipboard();
    const facebookAppId = env.get('facebookAppId');
    const githubAppId = env.get('githubAppId');
    return (react_1.default.createElement("section", { className: "section container-fluid" },
        react_1.default.createElement("h3", { className: "box-title" }, "Settings"),
        react_1.default.createElement("form", { className: "form-sm-horizontal", onSubmit: onSubmit },
            state === 'form-success' && (react_1.default.createElement(alert_1.default, { type: alert_1.AlertType.SUCCESS }, "Your settings have been saved.")),
            errors.form &&
                errors.form.map((message, i) => (react_1.default.createElement(alert_1.default, { key: i, type: alert_1.AlertType.DANGER }, message))),
            react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.TEXT, id: "settings-firstname", label: "Firstname", value: userSettings.firstname, errors: errors.firstname, onChange: onUpdateFirstname }),
            react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.TEXT, id: "settings-lastname", label: "Lastname", value: userSettings.lastname, errors: errors.lastname, onChange: onUpdateLastname }),
            react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.TEXT, id: "settings-username", label: "Username", value: userSettings.username, errors: errors.username, onChange: onUpdateUsername }),
            facebookAppId && (react_1.default.createElement("div", { className: "row mb-3" },
                react_1.default.createElement("label", { htmlFor: "settings-facebook", className: "col-sm-2 col-form-label" }, "Facebook"),
                react_1.default.createElement("div", { className: "col-sm-10" },
                    react_1.default.createElement("div", { className: "d-grid" }, (userSettings.facebookId && (react_1.default.createElement("button", { onClick: onRevokeFacebook, className: "btn btn-info" },
                        react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_brands_svg_icons_1.faFacebookF }),
                        " Revoke Facebook"))) || (react_1.default.createElement("a", { href: (0, auth_1.facebookLoginUrl)(facebookAppId), className: "btn btn-social btn-facebook" },
                        react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_brands_svg_icons_1.faFacebookF }),
                        " Connect with Facebook")))))),
            githubAppId && (react_1.default.createElement("div", { className: "row mb-3" },
                react_1.default.createElement("label", { htmlFor: "settings-github", className: "col-sm-2 col-form-label" }, "Github"),
                react_1.default.createElement("div", { className: "col-sm-10" },
                    react_1.default.createElement("div", { className: "d-grid" }, (userSettings.githubId && (react_1.default.createElement("button", { onClick: onRevokeGithub, className: "btn btn-info" },
                        react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_brands_svg_icons_1.faGithub }),
                        " Revoke Github"))) || (react_1.default.createElement("a", { href: (0, auth_1.githubLoginUrl)(githubAppId), className: "btn btn-social btn-github" },
                        react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_brands_svg_icons_1.faGithub }),
                        " Connect with Github")))))),
            react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.TEXT, id: "settings-apiKeyGenerate", label: "Api key", value: userSettings.apiKey, errors: errors.apiKey, onChange: onUpdateApiKey, groups: react_1.default.createElement("button", { type: "button", className: "input-group-text", onClick: onGenerateKey }, "Generate") }),
            react_1.default.createElement("div", { className: "row mb-3" },
                react_1.default.createElement("label", { htmlFor: "settings-apiKey", className: "col-sm-2 col-form-label" }, "Api usage"),
                react_1.default.createElement("div", { className: "col-sm-10" },
                    react_1.default.createElement("div", { className: "input-group" },
                        react_1.default.createElement("button", { type: "button", className: "input-group-text", onClick: onCopyApiUsage },
                            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: free_regular_svg_icons_1.faClipboard })),
                        react_1.default.createElement("input", { type: "text", className: "form-control", id: "settings-apiKey", value: clipboard || '', readOnly: true, placeholder: "api key" })))),
            react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.CHECKBOX, id: "settings-optinNewsletter", label: "Subscribe to the newsletter", value: leadSettings.optinNewsletter, errors: errors.optinNewsletter, onChange: onChangeOptinNewsletter }),
            react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.CHECKBOX, id: "settings-optinNewsletter", label: "Subscribe to blog updates", value: leadSettings.optinBlog, errors: errors.optinBlog, onChange: onChangeOptinBlog }),
            leadSettings.githubUsername && (react_1.default.createElement(form_input_1.default, { type: form_input_1.FormInputType.CHECKBOX, id: "settings-optinGithub", label: "Subscribe to github updates", value: leadSettings.optinGithub, errors: errors.optinGithub, onChange: onChangeOptinGithub })),
            react_1.default.createElement("div", { className: "row mb-3" },
                react_1.default.createElement("div", { className: "offset-sm-2 col-sm-10" },
                    react_1.default.createElement("div", { className: "d-grid" },
                        react_1.default.createElement("button", { type: "submit", className: "btn btn-primary", disabled: state === 'form-submit' }, "Save")))))));
};
exports.default = Settings;
